
package com.redxun.knowledge.map.controller;

import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.annotation.MethodDefine;
import com.redxun.common.annotation.ParamDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.constant.HttpMethodConstants;
import com.redxun.knowledge.map.entity.dao.Map;
import com.redxun.knowledge.map.entity.dto.MapDto;
import com.redxun.knowledge.map.entity.vo.MapVo;
import com.redxun.knowledge.map.service.MapServiceImpl;
import com.redxun.log.annotation.AuditLog;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/map")
@Api(tags = "业务--知识地图表")
@ClassDefine(title = "业务--知识地图表", alias = "MapController", path = "/api-knowledge/map",
        packages = "knowledge", packageName = "子系统名称")
public class MapController extends BaseController<Map> {

    @Autowired
    MapServiceImpl mapService;


    @Override
    public BaseService getBaseService() {
        return mapService;
    }

    @Override
    public String getComment() {
        return "业务--知识地图表";
    }

    @ApiOperation(value = "新增地图")
    @PostMapping("create")
    @AuditLog(
            operation = "保存数据"
    )
    @MethodDefine(
            title = "保存数据",
            path = "create",
            method = HttpMethodConstants.POST,
            params = {@ParamDefine(
                    title = "实体数据JSON",
                    varName = "mapDto"
            )}
    )
    public JsonResult create(@RequestBody MapDto mapDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String pkId = mapService.create(mapDto);
        return jsonResult.setData(pkId);
    }

    @ApiOperation("获取知识地图详情")
    @GetMapping("data")
    public JsonResult data(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        MapVo mapVo = mapService.data(pkId);
        return jsonResult.setData(mapVo);
    }

    @ApiOperation(value = "编辑地图")
    @PostMapping("edit")
    @AuditLog(
            operation = "保存数据"
    )
    @MethodDefine(
            title = "保存数据",
            path = "edit",
            method = HttpMethodConstants.POST,
            params = {@ParamDefine(
                    title = "实体数据JSON",
                    varName = "mapDto"
            )}
    )
    public JsonResult edit(@RequestBody MapDto mapDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String pkId = mapService.create(mapDto);
        return jsonResult.setData(pkId);
    }

    @ApiOperation("根据地图类型字典值获取知识地图列表")
    @GetMapping("listOfType")
    public JsonResult listOfType(@RequestParam("type") Integer type){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<java.util.Map<String, Object>> result = mapService.listOfType(type);
        return jsonResult.setData(result);
    }

    @ApiOperation("根据知识地图ID获取知识地图配置的学习路径列表")
    @GetMapping("routes")
    public JsonResult routes(@RequestParam("mapId") String mapId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<HashMap<String, Object>> result = mapService.routes(mapId);
        if (result != null){
            return jsonResult.setData(result);
        }else {
            return jsonResult.setData("您没有访问权限");
        }
    }

}

