
package com.redxun.knowledge.map.service;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.map.entity.dao.Map;
import com.redxun.knowledge.map.entity.dao.MapContent;
import com.redxun.knowledge.map.entity.dao.MapGroupPurviewUser;
import com.redxun.knowledge.map.entity.dao.MapRoute;
import com.redxun.knowledge.map.entity.vo.MapContentVo;
import com.redxun.knowledge.map.mapper.MapContentMapper;
import com.redxun.knowledge.map.mapper.MapGroupPurviewUserMapper;
import com.redxun.knowledge.map.mapper.MapMapper;
import com.redxun.knowledge.map.mapper.MapRouteMapper;
import com.redxun.knowledge.mapper.KnowledgeMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;

/**
* [业务--知识地图学习路径表]业务服务类
*/
@Service
public class MapRouteServiceImpl extends SuperServiceImpl<MapRouteMapper, MapRoute> implements BaseService<MapRoute> {

    @Resource
    private MapRouteMapper mapRouteMapper;

    @Autowired
    private MapMapper mapMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private MapContentMapper mapContentMapper;

    @Autowired
    private MapGroupPurviewUserMapper mapGroupPurviewUserMapper;

    @Autowired
    private KnowledgeMapper knowledgeMapper;

    @Override
    public BaseDao<MapRoute> getRepository() {
        return mapRouteMapper;
    }

    /**
     * PC端根据知识地图路径ID获取知识地图学习内容，根据tops参数设置请求内容条数。(权限判断)
     * @param routeId
     * @param tops
     * @return
     */
    public List<MapContentVo> contentTops(String routeId, Integer tops) {
        // 获取地图
        Map map = mapMapper.selectById(mapRouteMapper.selectById(routeId).getMapId());
        // 判断权限
        //当前登录用户的部门Id
        String deptId = userService.queryLoginUserDeptId();
        String[] groupIds = map.getGroupId().split(",");
        //包含登录人的部门
        if (Arrays.asList(groupIds).contains(deptId)) {
            if (map.getGroupPurview() == 1) {
                return contentInfo(routeId,tops);
            } else {
                QueryWrapper<MapGroupPurviewUser> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("MAP_ID", map.getPkId());
                queryWrapper.eq("USER_ID", userService.queryLoginUser());
                // 查询结果>0,说明该员工可见
                Integer count = mapGroupPurviewUserMapper.selectCount(queryWrapper);
                if (count > 0) {
                    return contentInfo(routeId,tops);
                }
            }
        }
        return null;
    }

    /**
     * PC端根据知识地图路径ID获取知识地图学习内容，根据tops参数设置请求内容条数(方法封装)
     * @param routeId
     * @return
     */
    private List<MapContentVo> contentInfo(String routeId,Integer tops) {
        List<MapContent> mapContentList = mapContentMapper.selectList(new LambdaQueryWrapper<MapContent>().eq(MapContent::getRouteId, routeId).orderByAsc(MapContent::getSort));
        List<MapContentVo> result = mapContentList.stream().limit(tops).map(mapContent -> {
            MapContentVo mapContentVo = new MapContentVo();
            BeanUtils.copyProperties(mapContent, mapContentVo);
            return mapContentVo;
        }).collect(Collectors.toList());
        return result;
    }
}
